#!/bin/bash
# 20190816 Robert Martin-Legene
# GPL2-only

# BFA tiene 3 bootnodes oficiales.
# Para levantar un bootnode hay que poner la clave en
# este archivo. Se genera la clave con: bootnode -genkey
bootnodekeyfile=${BFANETWORKDIR}/bootnode/key

# Bail out if anything fails.
trap "exit 1" ERR
# Detect children dying
trap "reaper" SIGINT SIGCHLD
unset LOGDIR LOGPIPE PIDIDX
declare -A PIDIDX
trap "killallprocs" SIGTERM

function reaper()
{
    local wecare=0
    for pid in ${!PIDIDX[*]}
    do
        kill -0 $pid 2>/dev/null && continue
        local rc=0
        wait $pid || rc=$? || true
        echo "*** ${PIDIDX[$pid]} (pid $pid) has exited with value $rc" 
        unset PIDIDX[$pid]
        wecare=1
    done
    test $wecare = 1 || return
    # kill all - an extra kill doesn't hurt. I hope.
    for pid in ${!PIDIDX[*]}
    do
        # don't kill log.sh, because it may still be logging for us
        if [ "${PIDIDX[$pid]}" != "log.sh" ]
        then
            echo "*** Killing ${PIDIDX[$pid]} (pid $pid)."
            kill $pid || true
        fi
    done
    max=30
}

function killallprocs()
{
    if [ ${#PIDIDX[*]} -gt 0 ]
    then
        echo "*** Killing all remaining processes: ${PIDIDX[*]} (${!PIDIDX[*]})."
        kill -KILL ${!PIDIDX[*]} 2>/dev/null || true
    fi
}

# You can start as:
# BFAHOME=/home/bfa/bfa singlestart.sh
# singlestart.sh /home/bfa/bfa
if [ -z "${BFAHOME}" -a -n "$1" -a -f "$1" ]
then
    export BFAHOME="$1"
fi
if [ -z "${BFAHOME}" ]; then echo "\$BFAHOME not set. Did you source `dirname $0`/env ?" >&2; exit 1; fi
#
if [ -r "${BFAHOME}/bin/env" ]
then
    source ${BFAHOME}/bin/env
else
    if [ "$VIRTUALIZATION" = "DOCKER" ]
    then
        # The idea is that the environment already is set by the Docker environment.
        touch ${BFAHOME}/bin/env
    else
        echo "Can't do much without a \$BFAHOME/bin/env file. Maybe you can copy one of these?" >&2
        ls -1 ${BFAHOME}/*/env >&2
        exit 1
    fi
fi
source ${BFAHOME}/bin/libbfa.sh

if [ "$VIRTUALIZATION" = "DOCKER" ]
then
    echo "See log info with \"docker logs\""
else
    echo "Logging mostly everything to ${BFANODEDIR}/log"
    echo "Consider running: tail -n 1000 -F ${BFANODEDIR}/log"
    echo "or: bfalog.sh"

    # Docker has it's own logging facility, so we will not use our own
    # logging functionality if we're in docker.
    echo "*** Setting up logging."
    # Clean up logging
    LOGDIR=$( mktemp -d )
    trap "rm -rf ${LOGDIR}" EXIT
    LOGPIPE=${LOGDIR}/logpipe
    mknod ${LOGPIPE} p
    ${BFAHOME}/bin/log.sh ${BFANODEDIR}/log < ${LOGPIPE} &
    PIDIDX[$!]="log.sh"
    exec > ${LOGPIPE} 2>&1
fi

echo "*** Starting geth."
# "NoPruning=true" means "--gcmode archive"
geth --config ${BFATOML} &
PIDIDX[$!]="geth"

# bootnode
if [ -r "$bootnodekeyfile" ]
then
    echo "*** Starting bootnode."
    bootnode --nodekey $bootnodekeyfile &
    PIDIDX[$!]="bootnode"
fi

echo "*** Starting monitor.js"
monitor.js &
PIDIDX[$!]="monitor.js"

max=-1
# Exit if only 1 process remains - we hope it is log.sh, but either way,
# it is time to end.
while [ "${#PIDIDX[*]}" -gt 1 -a $max -ne 0 ]
do
    sleep 1
    max=$(( $max - 1 ))
done
killallprocs
